@@ -191,7 +191,7 @@ class ZookeeperClient(object):
             d.errback(NotConnectedException("not connected"))
             return d
 
-    def _check_result(self, result_code, deferred, extra_codes=()):
+    def _check_result(self, result_code, deferred, extra_codes=(), path=None):
         """Check an API call or result for errors.
 
         :param result_code: The api result code.
@@ -204,6 +204,8 @@ class ZookeeperClient(object):
         error = None
         if not result_code == zookeeper.OK and not result_code in extra_codes:
             error_msg = zookeeper.zerror(result_code)
+            if path is not None:
+                error_msg += " %s" % path
             error_class = ERROR_MAPPING.get(
                 result_code, zookeeper.ZooKeeperException)
             error = error_class(error_msg)
@@ -230,14 +232,14 @@ class ZookeeperClient(object):
             return d
 
         def _cb_get(result_code, value, stat):
-            if self._check_result(result_code, d):
+            if self._check_result(result_code, d, path=path):
                 return
             d.callback((value, stat))
 
         callback = self._zk_thread_callback(_cb_get)
         watcher = self._wrap_watcher(watcher, "get", path)
         result = zookeeper.aget(self.handle, path, watcher, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _get_children(self, path, watcher):
@@ -246,14 +248,14 @@ class ZookeeperClient(object):
             return d
 
         def _cb_get_children(result_code, children):
-            if self._check_result(result_code, d):
+            if self._check_result(result_code, d, path=path):
                 return
             d.callback(children)
 
         callback = self._zk_thread_callback(_cb_get_children)
         watcher = self._wrap_watcher(watcher, "child", path)
         result = zookeeper.aget_children(self.handle, path, watcher, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _exists(self, path, watcher):
@@ -263,14 +265,14 @@ class ZookeeperClient(object):
 
         def _cb_exists(result_code, stat):
             if self._check_result(
-                result_code, d, extra_codes=(zookeeper.NONODE,)):
+                result_code, d, extra_codes=(zookeeper.NONODE,), path=path):
                 return
             d.callback(stat)
 
         callback = self._zk_thread_callback(_cb_exists)
         watcher = self._wrap_watcher(watcher, "exists", path)
         result = zookeeper.aexists(self.handle, path, watcher, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _wrap_watcher(self, watcher, watch_type, path):
@@ -405,7 +407,7 @@ class ZookeeperClient(object):
         """
         self.connected = False
 
-        if not self.handle:
+        if self.handle is None:
             return
 
         try:
@@ -522,11 +524,11 @@ class ZookeeperClient(object):
             self._cb_created, d, data, acls, flags)
         result = zookeeper.acreate(
             self.handle, path, data, acls, flags, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _cb_created(self, d, data, acls, flags, result_code, path):
-        if self._check_result(result_code, d):
+        if self._check_result(result_code, d, path=path):
             return
         d.callback(path)
 
@@ -541,16 +543,14 @@ class ZookeeperClient(object):
         @param version: the integer version of the node.
         """
         d = defer.Deferred()
-        if self._check_connected(d):
-            return d
 
         callback = self._zk_thread_callback(self._cb_deleted, d, path)
         result = zookeeper.adelete(self.handle, path, version, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _cb_deleted(self, d, path, result_code):
-        if self._check_result(result_code, d):
+        if self._check_result(result_code, d, path=path):
             return
         d.callback(result_code)
 
@@ -652,13 +652,13 @@ class ZookeeperClient(object):
             return d
 
         def _cb_get_acl(result_code, acls, stat):
-            if self._check_result(result_code, d):
+            if self._check_result(result_code, d, path=path):
                 return
             d.callback((acls, stat))
 
         callback = self._zk_thread_callback(_cb_get_acl)
         result = zookeeper.aget_acl(self.handle, path, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def set_acl(self, path, acls, version=-1):
@@ -691,11 +691,11 @@ class ZookeeperClient(object):
         callback = self._zk_thread_callback(self._cb_set_acl, d, path, acls)
         result = zookeeper.aset_acl(
             self.handle, path, version, acls, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _cb_set_acl(self, d, path, acls, result_code):
-        if self._check_result(result_code, d):
+        if self._check_result(result_code, d, path=path):
             return
         d.callback(result_code)
 
@@ -716,11 +716,11 @@ class ZookeeperClient(object):
 
         callback = self._zk_thread_callback(self._cb_set, d, path, data)
         result = zookeeper.aset(self.handle, path, data, version, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
 
     def _cb_set(self, d, path, data, result_code, node_stat):
-        if self._check_result(result_code, d):
+        if self._check_result(result_code, d, path=path):
             return
         d.callback(node_stat)
 
@@ -796,11 +796,11 @@ class ZookeeperClient(object):
             return d
 
         def _cb_sync(result_code, path):
-            if self._check_result(result_code, d):
+            if self._check_result(result_code, d, path=path):
                 return
             d.callback(path)
 
         callback = self._zk_thread_callback(_cb_sync)
         result = zookeeper.async(self.handle, path, callback)
-        self._check_result(result, d)
+        self._check_result(result, d, path=path)
         return d
